#!/usr/bin/env /proj/axaf/bin/perlwrap
BEGIN
{
    $ENV{SYBASE} = "/soft/sybase";
    $ENV{LANG} = "";
}

use strict;
use DBI;
use DBD::Sybase;
use Astro::ADS::Query;
use Astro::ADS::Result;
use Astro::ADS::Result::Paper;

use vars qw($dbh1 $dsn1 $password %param $VERSION);
$VERSION = '$Id: getCitations.pl,v 1.9 2004/11/05 18:52:42 arcops Exp $';

{
    use Getopt::Long;
    parse_opts();
    
    if ($param{version})
    {
        print $VERSION, "\n";
        exit( 0 );
    }
    
    if ($param{help}) 
    { 
        usage(0);
    }

    use Term::ReadKey;
    if ($param{cron})
    {
        open (PWD, "pwd_file") ||
            die "Sorry, can't open pwd_file: $!\n";
        my @pwds = <PWD>;
        close PWD;
        $password = $pwds[0];
        chomp $password;
    }
    else
    {
        ReadMode 'noecho';
        if ($param{S} !~ /production_server/)
        {
            print "Enter password for $param{U} on $param{S}: \n";
            $password = ReadLine( 0 );
            chomp $password;
        }
        else
        {
            print "Enter password for $param{U} on $param{S}: \n";
            $password = ReadLine( 0 );
            chomp $password;
        }
        ReadMode 'restore';
    }

    my $server = "server=$param{S}";
    my $db = "database=bibcodes";
    my $script_name = "scriptName=getCitations.pl";

    # Database connection 1
    $dsn1 = "DBI:Sybase:$server;$script_name;$db";
    $dbh1 = DBI->connect($dsn1, "$param{U}", $password, {
        PrintError => 0,
        RaiseError => 1});

    open(LOG, ">/LOGS/cite_cnt.log") ||
	die "Sorry, couldn't open /LOGS/cite_cnt.log: $!\n";
    print LOG scalar localtime(), "\n";

    my @aux_bibcodes;
    my $sqlh1 = $dbh1->prepare(qq(select bibcode, ads_db from bibcodes_main 
				  where bib_status = null 
				  order by bibcode));
    $sqlh1->execute();
    while (my ($bibcode, $ads_db) = $sqlh1->fetchrow_array) {
	push @aux_bibcodes, "$bibcode&db_key=$ads_db";
    }
    $sqlh1->finish;

    my $cite_update = $dbh1->prepare(qq(update bibcodes_main set cite_cnt = ?
					where bibcode = ?));
    print LOG "bibcodes..bibcodes_main\n";
    close LOG;
    my $total = scalar @aux_bibcodes;
    my $done = 0;
    foreach my $bibq (@aux_bibcodes) {
	my ($bibcode, $stuff) = split /&/, $bibq;

	open(LOG, ">>/data/mola/chandracite/LOGS/cite_cnt.log") ||
	    die "Sorry, couldn't open /data/mola/chandracite/LOGS/cite_cnt.log: $!\n";
	print LOG "$bibq\n" if $param{full_log};

	print LOG "Zero cite_cnt: ", scalar localtime if $param{full_log};;
	$cite_update->execute(0, $bibcode);
	$cite_update->finish;
	print LOG " ", scalar localtime, "\n" if $param{full_log};;

	print LOG "ADS query: ", scalar localtime if $param{full_log};;
	my $query = new Astro::ADS::Query(Bibcode => $bibq);
	$query->timeout(60);
	eval{ $query->url("adsabs.harvard.edu") };

	if ($@) {
	    eval{ $query->url("adsabs.harvard.edu") };
	    print_error($bibcode, $@, *LOG) if $@;
	}
	else {
	    my $result;
	    eval{ $result = $query->querydb() };
	    if ($@) {
		eval{ $result = $query->querydb() };
		print_error($bibcode, $@, *LOG) if $@;
	    }
	    else {
		my $paper = $result->poppaper();
		print LOG " ", scalar localtime, "\n" if $param{full_log};;
		
		if (!$paper) {
		    print LOG "$bibcode: bibcode not found\n";
		    print "$bibcode: bibcode not found\n";
		}
		else {
		    my @outbound_links = $paper->links();
		    foreach my $link (@outbound_links) {
			if ($link =~ /CITATIONS/) {
			    my $cite_results;

			    print LOG "ADS query: ", scalar localtime 
				if $param{full_log};;
			    my $cite_query = 
				new Astro::ADS::Query(Bibcode => $bibq);
			    $cite_query->timeout(60);
			    eval{ $cite_query->url("adsabs.harvard.edu")};
			    if ($@) {
				eval{ $cite_query->url("adsabs.harvard.edu")};
				print_error($bibcode, $@, *LOG) if $@;
			    }
			    else {
				eval{$cite_results = 
					 $cite_query->followup($bibq, 
							       'CITATIONS')};
				if ($@) {
				    eval{$cite_results = 
					     $cite_query->
						 followup($bibq, 
							  'CITATIONS')};
				    print_error($bibcode, $@, *LOG) if $@;
				}
				else {
				    my $num_cites = $cite_results->sizeof();
				    print LOG " ", scalar localtime, "\n" 
					if $param{full_log};
				    print LOG "cite_cnt: ", scalar localtime 
					if $param{full_log};
				    $cite_update->execute($num_cites, 
							  $bibcode);
				    $cite_update->finish;
				    $done++;
				    print LOG " ", scalar localtime, "\n" 
					if $param{full_log};;
				}
			    }
			}
		    }
		}
	    }
	}
	close LOG;
    }

    $dbh1->disconnect;
    print LOG "Updated $done citation counts of $total\n";
    print LOG scalar localtime(), "\n";
}

#******************************************************************************
# Subroutine to print errors
#******************************************************************************
sub print_error {
    my ($bibcode, $error, $log) = @_;
    print "$bibcode: $error\n";
    print $log "$bibcode: $@\n" if ($error);
}

#******************************************************************************
# Subroutine for parse opts
#******************************************************************************
sub parse_opts
{

  %param = (
            U => undef,
	    S => undef,
            verbose => 0
           );

  GetOptions( \%param,
              "U=s",
	      "S=s",
	      "full_log",
              "cron",
              "verbose",
              "version",
              "help"
            ) or exit(1);

  return if $param{help} or $param{version};


  my $err = 0;
  while ( my ( $par, $val ) = each ( %param ) )
  {
    next if defined $val;
    warn("parameter `$par' not set\n");
    $err++;
  }

  exit(1) if $err;

}

#******************************************************************************
# Subroutine for usage statements
#******************************************************************************
sub usage
{
  my ( $exit ) = @_;

  local $^W = 0;
  require Pod::Text;
  Pod::Text::pod2text( '-75', $0 );
  exit $exit;
}

__END__

=head1 USAGE

getCitations.pl [options]

=head1 OPTIONS

B<getCitations.pl> uses long option names.  You can type as few characters as
are necessary to match the option name.

=over 4

=item B<-U> (user)

username for logging onto sql server

=item B<-S> (server)

name of sql server

=item B<-bibcode>

when flag is set, each bibcode checked is printed to STDOUT

=item B<-help>

displays documentation for B<getCitations.pl>

=item B<-version>

displays the version

=item B<-verbose>

displays required options

=back

=head1 DESCRIPTION

This script updates the cite_cnt column for each bicode in
bibcodes..bibcodes_main.  The cite_cnt is retrieved from ADS.

=head1 AUTHOR

Sherry L. Winkelman
